# Files Map -- Crosswalk map.

# Extenions:        .yaml, .json, .csv., .xlsx, .png
# Sub-extensions:   .Topic.yaml, .Comment.yaml, .Drawing.png

# In order to hard-assign files with sub-extensions, we need a way to link sub-extensions with Wikis for concepts.

# Example:
'''
*GH~mindey.terms.topic_metaculus.* <=> GH:mindey/terms/topic#metaculus
*WD~Q123.*                         <=> WD:Q123
'''

FMAP = {}

def f2n(token):
    'filename to name'

    # PARSE
    if '_' in token:
        _concept, _format = token.rsplit('_', 1)
    else:
        _concept, _format = token, None

    if '~' in _concept:
        _namespace, _alias = _concept.rsplit('~', 1)
    else:
        _namespace, _alias = _concept, None

    # MERGE
    name = ''

    if _namespace == 'GH':            # github wikis
        name += _namespace + ':'
    elif _namespace == 'GHF':         # github repo files
        name += _namespace + ':'
    elif _namespace == 'WD':          # wikidata
        name += _namespace + ':'

    if _alias is not None:
        name += _alias.replace('.', '/')

    if _format is not None:
        name += '#'+_format

    return name


def n2f(token):
    'name to filename'

    # PARSE
    if '#' in token:
        _concept, _format = token.rsplit('#', 1)
    else:
        _concept, _format = token, None

    if ':' in _concept:
        _namespace, _path = _concept.rsplit(':', 1)
    else:
        _namespace, _path = _concept, None

    # MERGE
    name = ''

    name += _namespace + '~'

    if _path is not None:
        name += _path.replace('/', '.')

    if _format is not None:
        name += '_'+_format

    return name



from metawiki import data

NAMESPACES = {}

# Wikidata (https://www.wikidata.org/wiki/{reference})
# WD: namespace refered by keys starting with 'WD:'
WIKIDATA = lambda N: {
    '%s:Q{integer}' % (N,): [
        'https://www.wikidata.org/wiki/Q{integer}'
    ],
    '%s:P{integer}' % (N,): [
        'https://www.wikidata.org/wiki/Property:P{integer}'
    ]
}
NAMESPACES['WD:'] = WIKIDATA('WD')

# Github (https://github.com/{user}/{repo}/wiki/{concept})
# GH: wiki namespace, refered by keys starting with 'GH:'
GITHUB_WIKI = lambda N: {
    '%s:{user}/{repo}/{concept}' % (N,): [
        'https://github.com/{user}/{repo}/wiki/{concept}',
        'https://raw.githubusercontent.com/wiki/{user}/{repo}/{concept}.md',
    ],
    '%s:{user}/{repo}/{concept}#{format}' % (N,): [
        'https://github.com/{user}/{repo}/wiki/{concept}#{format}',
        'https://raw.githubusercontent.com/wiki/{user}/{repo}/{concept}.md#{format}',
    ]
}
NAMESPACES['GH:'] = GITHUB_WIKI('GH')

# Github files (https://github.com/{user}/{repo}/blob/master/{concept})
# GHF: refered by keys starting with 'GHF:'
GITHUB_FILE = lambda N: {
    '%s:{user}/{repo}/{concept}' % (N,): [
        'https://github.com/{user}/{repo}/blob/master/{concept}',
        'https://raw.githubusercontent.com/{user}/{repo}/{concept}',
    ],
    '%s:{user}/{repo}/{concept}#{format}' % (N,): [
        'https://github.com/{user}/{repo}/blob/master/{concept}#{format}',
        'https://raw.githubusercontent.com/{user}/{repo}/{concept}#{format}',
    ]
}
NAMESPACES['GHF:'] = GITHUB_FILE('GHF') # FILEs of ooio repos


# LOV vocabularies (https://lov.linkeddata.es/dataset/lov/vocabs/)
# Close to 700 namespaces.
LOV = data.get_lov()
RESERVED = NAMESPACES.keys()

for vocab in LOV:
    if vocab['prefix'] not in RESERVED:
        NAMESPACES['%s:' % vocab['prefix']] = {
            '%s:{concept}' % vocab['prefix']: ['%s{concept}' % vocab['nsp']]
        }


MAP = {}

for key, value in NAMESPACES.items():
    MAP = dict(MAP, **value)



from .fmap import (
    n2f,
    f2n,
)
from metawiki import (
    ext2url,
    url2ext,
)

def test_f2n():

    # GH:
    assert f2n('GH~mindey.-.topic_metaculus') == 'GH:mindey/-/topic#metaculus'

    # WD:
    assert f2n('WD~Q123') == 'WD:Q123'

def test_n2f():

    # GH:
    assert n2f('GH:mindey/-/topic#metaculus') == 'GH~mindey.-.topic_metaculus'

    # WD:
    assert n2f('WD:Q123') == 'WD~Q123'

def test_url2f():
    assert url2ext('https://github.com/mindey/-/wiki/cucumber#xtrandard') == 'GH~mindey.-.cucumber_xtrandard'

    assert url2ext('https://github.com/infamily/_/wiki/topic#google-plus') == 'GH~infamily._.topic_google-plus'

def test_f2url():
    assert 'https://github.com/mindey/-/wiki/cucumber#xtrandard' == ext2url('GH~mindey.-.cucumber_xtrandard')

    assert 'https://github.com/infamily/_/wiki/topic#google-plus' == ext2url('GH~infamily._.topic_google-plus')



from metawiki import (
    url_to_name,
    name_to_url
)

def test_to_name():

    # GH:
    result = url_to_name(
        'https://github.com/infamily/-/wiki/percent-change#1hourly')

    assert result == 'GH:infamily/-/percent-change#1hourly'

    # GHF:
    result = url_to_name(
        'https://github.com/mindey/-/blob/master/keys.md#a225d1e4')

    assert result == 'GHF:mindey/-/keys.md#a225d1e4'

    # WD:
    result =  url_to_name('https://www.wikidata.org/wiki/Q1347367')

    assert result == 'WD:Q1347367'

    result =  url_to_name('https://www.wikidata.org/wiki/Property:P31')

    assert result == 'WD:P31'

def test_to_url():

    # GH:
    result = name_to_url('GH:infamily/-/level')

    assert result == 'https://github.com/infamily/-/wiki/level'

    # GHF:
    result = name_to_url('GHF:mindey/-/keys.md#a225d1e4')

    assert result == 'https://github.com/mindey/-/blob/master/keys.md#a225d1e4'

    # WD:
    result = name_to_url('WD:Q1347367')

    assert result == 'https://www.wikidata.org/wiki/Q1347367'
